package com.elitesland.router.service;

import com.alibaba.nacos.api.config.ConfigService;
import com.elitesland.router.bean.GatewayRouteDefinition;
import com.elitesland.router.util.GateWayNacosUtils;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.stereotype.Service;


/*
 *@Author Horbin.Zhang
 *@Date 2021-02-12 14:17
 *@Description
 */
@Service
public class GatewayRoutesInNacosConfigService {

    @Autowired
    private GateWayNacosUtils gateWayNacosUtils;

    @Autowired
    org.springframework.cloud.gateway.config.PropertiesRouteDefinitionLocator propertiesRouteDefinitionLocator;

    @Autowired
    private DynamicRouteService dynamicRouteService;

    RouteLocatorBuilder routeLocatorBuilder;


    public GatewayRouteDefinition addOrUpdateRouteDefinitiToNacosServer(GatewayRouteDefinition gwdefinition) {
        try {
            Gson gson = new Gson();
            String gateWayDefinitions = gateWayNacosUtils.getRouteDefinitionsFromNacosServer();
            boolean flag = false;
            if (null != gateWayDefinitions) {
                JsonArray jsonArray = gson.fromJson(gateWayDefinitions, JsonArray.class);
                JsonArray newConfig = new JsonArray();
                if (jsonArray.size() > 0) {
                    jsonArray.forEach(routeDefinitionAsJsonObject -> {
                        GatewayRouteDefinition gatewayRouteDefinition = gson.fromJson(routeDefinitionAsJsonObject, GatewayRouteDefinition.class);
                        if (null != gatewayRouteDefinition) {
                            if (!gatewayRouteDefinition.getId().equalsIgnoreCase(gwdefinition.getId())) {
                                newConfig.add(routeDefinitionAsJsonObject);
                            }
                        }
                    });
                }
                    JsonElement jsonElement = gson.toJsonTree(gwdefinition, GatewayRouteDefinition.class);
                    newConfig.add(jsonElement);
                ConfigService configService = gateWayNacosUtils.getNacosConfigService();
                String group = gateWayNacosUtils.getNacosConfigGroup();
                String routeConfigDataId = gateWayNacosUtils.getNacosConfigDataIdForGateWayDefinition();
                configService.publishConfig(routeConfigDataId, group, newConfig.toString(), "json");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return gwdefinition;
    }

    public String deleteRouteDefinitiToNacosServer(String routeId) {
        try {
            Gson gson = new Gson();
            String gateWayDefinitions = gateWayNacosUtils.getRouteDefinitionsFromNacosServer();
            if (null != gateWayDefinitions) {
                JsonArray jsonArray = gson.fromJson(gateWayDefinitions, JsonArray.class);
                JsonArray newConfig = new JsonArray();
                if (jsonArray.size() > 0) {
                    jsonArray.forEach(routeDefinitionAsJsonObject -> {
                        GatewayRouteDefinition gatewayRouteDefinition = gson.fromJson(routeDefinitionAsJsonObject, GatewayRouteDefinition.class);
                        if (null != gatewayRouteDefinition) {
                            if (!gatewayRouteDefinition.getId().equalsIgnoreCase(routeId)) {
                                newConfig.add(routeDefinitionAsJsonObject);
                            }
                        }
                    });
                }
                ConfigService configService = gateWayNacosUtils.getNacosConfigService();
                String group = gateWayNacosUtils.getNacosConfigGroup();
                String routeConfigDataId = gateWayNacosUtils.getNacosConfigDataIdForGateWayDefinition();
                configService.publishConfig(routeConfigDataId, group, newConfig.toString(), "json");
            }
            return "删除成功！";
        } catch (Exception e) {
            e.printStackTrace();
            return "删除失败！";
        }
    }

    public JsonArray getGatewayDefinitionsFromNacosServer() {
        JsonArray jsonArrayOfJsonGatewayDefinitions = null;
        try {
            Gson gson = new Gson();
            String gateWayDefinitions = gateWayNacosUtils.getRouteDefinitionsFromNacosServer();
            if (null != gateWayDefinitions) {
                jsonArrayOfJsonGatewayDefinitions = gson.fromJson(gateWayDefinitions, JsonArray.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return jsonArrayOfJsonGatewayDefinitions;
    }
}
